@@ -33,8 +33,9 @@ decode(struct file_contents *fc, struct ContentInfo **result)
 	struct ContentInfo *cinfo;
 	int error;
 
+	/* Validate DER encoding rfc6488#section3 bullet 1.l */
 	error = asn1_decode_fc(fc, &asn_DEF_ContentInfo, (void **) &cinfo,
-	    true);
+	    true, true);
 	if (error)
 		return error;
 
