@@ -100,6 +100,9 @@ handle_sdata_certificate(ANY_t *cert_encoded, struct signed_object_args *args,
 		goto end2;
 	error = certificate_validate_extensions_ee(cert, sid, &args->refs,
 	    &policy);
+	if (error)
+		goto end2;
+	error = certificate_validate_aia(args->refs.caIssuers, cert);
 	if (error)
 		goto end2;
 	error = certificate_validate_signature(cert, signedData, signature);
@@ -128,7 +131,7 @@ validate_content_type_attribute(CMSAttributeValue_t *value,
 	int error;
 
 	error = asn1_decode_any(value, &asn_DEF_OBJECT_IDENTIFIER,
-	    (void **) &attrValues, true);
+	    (void **) &attrValues, true, false);
 	if (error)
 		return error;
 	eContentType = &eci->eContentType;
@@ -151,7 +154,7 @@ validate_message_digest_attribute(CMSAttributeValue_t *value,
 		return pr_err("There's no content being signed.");
 
 	error = asn1_decode_any(value, &asn_DEF_MessageDigest,
-	    (void **) &digest, true);
+	    (void **) &digest, true, false);
 	if (error)
 		return error;
 
@@ -404,7 +407,7 @@ signed_data_decode_pkcs7(ANY_t *coded, struct SignedData **result)
 	int error;
 
 	error = asn1_decode_any(coded, &asn_DEF_SignedDataPKCS7,
-	    (void **) &sdata_pkcs7, true);
+	    (void **) &sdata_pkcs7, true, false);
 	if (error)
 		return error;
 
@@ -417,7 +420,7 @@ signed_data_decode_pkcs7(ANY_t *coded, struct SignedData **result)
 	/* Parse content as OCTET STRING */
 	error = asn1_decode_any(sdata_pkcs7->encapContentInfo.eContent,
 	    &asn_DEF_ContentTypePKCS7,
-	    (void **) &sdata->encapContentInfo.eContent, true);
+	    (void **) &sdata->encapContentInfo.eContent, true, false);
 	if (error)
 		goto release_sdata;
 
@@ -451,10 +454,8 @@ signed_data_decode(struct signed_data *sdata, ANY_t *coded)
 
 	sdata->encoded = coded;
 
-	/* rfc6488#section-3.1.l */
-	/* TODO (next iteration) this is BER, not guaranteed to be DER. */
 	error = asn1_decode_any(coded, &asn_DEF_SignedData,
-	    (void **) &sdata->decoded, false);
+	    (void **) &sdata->decoded, false, false);
 	if (error) {
 		/* Try to decode as PKCS content (RFC 5652 section 5.2.1) */
 		error = signed_data_decode_pkcs7(coded, &sdata->decoded);
@@ -517,7 +518,7 @@ get_content_type_attr(struct SignedData *sdata, OBJECT_IDENTIFIER_t **result)
 				return -EINVAL;
 			return asn1_decode_any(attr->attrValues.list.array[0],
 			    &asn_DEF_OBJECT_IDENTIFIER,
-			    (void **) result, true);
+			    (void **) result, true, false);
 		}
 	}
 
