@@ -16,8 +16,10 @@
 #include "config/boolean.h"
 #include "config/incidences.h"
 #include "config/str.h"
+#include "config/sync_strategy.h"
 #include "config/uint.h"
 #include "config/uint32.h"
+#include "config/work_offline.h"
 
 /**
  * To add a member to this structure,
@@ -34,11 +36,11 @@ struct rpki_config {
 	char *tal;
 	/** Path of our local clone of the repository */
 	char *local_repository;
-	/** Synchronization (currently only RSYNC) download strategy. */
-	enum sync_strategy sync_strategy;
+	/** FIXME (later) Deprecated, remove it. RSYNC download strategy. */
+	enum rsync_strategy sync_strategy;
 	/**
 	 * Handle TAL URIs in random order?
-	 * (https://tools.ietf.org/html/rfc7730#section-3, last
+	 * (https://tools.ietf.org/html/rfc8630#section-3, last
 	 * paragraphs)
 	 */
 	bool shuffle_tal_uris;
@@ -51,6 +53,11 @@ struct rpki_config {
 	char *slurm;
 	/* Run as RTR server or standalone validation */
 	enum mode mode;
+	/*
+	 * Disable outgoing requests (currently rsync and http supported), if
+	 * 'true' uses only local files located at local-repository.
+	 */
+	bool work_offline;
 
 	struct {
 		/** The bound listening address of the RTR server. */
@@ -70,6 +77,22 @@ struct rpki_config {
 	} server;
 
 	struct {
+		/* Enables the protocol */
+		bool enabled;
+		/*
+		 * Priority, this will override the order set at the CAs in
+		 * their accessMethod extension.
+		 */
+		unsigned int priority;
+		/* Synchronization download strategy. */
+		enum rsync_strategy strategy;
+		/* Retry conf, utilized on errors */
+		struct {
+			/* Maximum number of retries on error */
+			unsigned int count;
+			/* Interval (in seconds) between each retry */
+			unsigned int interval;
+		} retry;
 		char *program;
 		struct {
 			struct string_array flat;
@@ -77,6 +100,36 @@ struct rpki_config {
 		} args;
 	} rsync;
 
+	struct {
+		/* Enables the protocol */
+		bool enabled;
+		/*
+		 * Priority, this will override the order set at the CAs in
+		 * their accessMethod extension.
+		 */
+		unsigned int priority;
+		/* Retry conf, utilized on errors */
+		struct {
+			/* Maximum number of retries on error */
+			unsigned int count;
+			/* Interval (in seconds) between each retry */
+			unsigned int interval;
+		} retry;
+	} rrdp;
+
+	struct {
+		/* User-Agent header set at requests */
+		char *user_agent;
+		/* Timeout in seconds for the connect phase */
+		unsigned int connect_timeout;
+		/* Maximum allowed time that a request can take */
+		unsigned int transfer_timeout;
+		/* Maximum idle time during a request */
+		unsigned int idle_timeout;
+		/* Directory where CA certs to verify peers are found */
+		char *ca_path;
+	} http;
+
 	struct {
 		/** Print ANSI color codes? */
 		bool color;
@@ -94,6 +147,9 @@ struct rpki_config {
 		/** File where the validated BGPsec certs will be stored */
 		char *bgpsec;
 	} output;
+
+	/* ASN1 decoder max stack size allowed */
+	unsigned int asn1_decode_max_stack;
 };
 
 static void print_usage(FILE *, bool);
@@ -173,9 +229,9 @@ static const struct option_field options[] = {
 		.name = "sync-strategy",
 		.type = &gt_sync_strategy,
 		.offset = offsetof(struct rpki_config, sync_strategy),
-		.doc = "RSYNC download strategy",
+		.doc = "RSYNC download strategy. Will be deprecated, use 'rsync.strategy' instead.",
 	}, {
-		.id = 2000,
+		.id = 2001,
 		.name = "shuffle-uris",
 		.type = &gt_bool,
 		.offset = offsetof(struct rpki_config, shuffle_tal_uris),
@@ -206,6 +262,12 @@ static const struct option_field options[] = {
 		.type = &gt_mode,
 		.offset = offsetof(struct rpki_config, mode),
 		.doc = "Run mode: 'server' (run as RTR server), 'standalone' (run validation once and exit)",
+	}, {
+		.id = 1005,
+		.name = "work-offline",
+		.type = &gt_work_offline,
+		.offset = offsetof(struct rpki_config, work_offline),
+		.doc = "Disable all outgoing requests (rsync, http (implies RRDP)) and work only with local repository files.",
 	},
 
 	/* Server fields */
@@ -301,6 +363,42 @@ static const struct option_field options[] = {
 	/* RSYNC fields */
 	{
 		.id = 3000,
+		.name = "rsync.enabled",
+		.type = &gt_bool,
+		.offset = offsetof(struct rpki_config, rsync.enabled),
+		.doc = "Enables RSYNC execution",
+	}, {
+		.id = 3001,
+		.name = "rsync.priority",
+		.type = &gt_uint32,
+		.offset = offsetof(struct rpki_config, rsync.priority),
+		.doc = "Priority of execution to fetch repositories files, a higher value means higher priority",
+		.min = 0,
+		.max = 100,
+	},{
+		.id = 3002,
+		.name = "rsync.strategy",
+		.type = &gt_rsync_strategy,
+		.offset = offsetof(struct rpki_config, rsync.strategy),
+		.doc = "RSYNC download strategy",
+	}, {
+		.id = 3003,
+		.name = "rsync.retry.count",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, rsync.retry.count),
+		.doc = "Maximum amount of retries whenever there's an RSYNC error",
+		.min = 0,
+		.max = UINT_MAX,
+	}, {
+		.id = 3004,
+		.name = "rsync.retry.interval",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, rsync.retry.interval),
+		.doc = "Period (in seconds) to wait between retries after an RSYNC error ocurred",
+		.min = 0,
+		.max = UINT_MAX,
+	},{
+		.id = 3005,
 		.name = "rsync.program",
 		.type = &gt_string,
 		.offset = offsetof(struct rpki_config, rsync.program),
@@ -308,14 +406,14 @@ static const struct option_field options[] = {
 		.arg_doc = "<path to program>",
 		.availability = AVAILABILITY_JSON,
 	}, {
-		.id = 3001,
+		.id = 3006,
 		.name = "rsync.arguments-recursive",
 		.type = &gt_string_array,
 		.offset = offsetof(struct rpki_config, rsync.args.recursive),
 		.doc = "RSYNC program arguments that will trigger a recursive RSYNC",
 		.availability = AVAILABILITY_JSON,
 	}, {
-		.id = 3002,
+		.id = 3007,
 		.name = "rsync.arguments-flat",
 		.type = &gt_string_array,
 		.offset = offsetof(struct rpki_config, rsync.args.flat),
@@ -323,6 +421,83 @@ static const struct option_field options[] = {
 		.availability = AVAILABILITY_JSON,
 	},
 
+	/* RRDP fields */
+	{
+		.id = 10000,
+		.name = "rrdp.enabled",
+		.type = &gt_bool,
+		.offset = offsetof(struct rpki_config, rrdp.enabled),
+		.doc = "Enables RRDP execution",
+	}, {
+		.id = 10001,
+		.name = "rrdp.priority",
+		.type = &gt_uint32,
+		.offset = offsetof(struct rpki_config, rrdp.priority),
+		.doc = "Priority of execution to fetch repositories files, a higher value means higher priority",
+		.min = 0,
+		.max = 100,
+	}, {
+		.id = 10002,
+		.name = "rrdp.retry.count",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, rrdp.retry.count),
+		.doc = "Maximum amount of retries whenever there's an error fetching RRDP files",
+		.min = 0,
+		.max = UINT_MAX,
+	}, {
+		.id = 10003,
+		.name = "rrdp.retry.interval",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, rrdp.retry.interval),
+		.doc = "Period (in seconds) to wait between retries after an error ocurred fetching RRDP files",
+		.min = 0,
+		.max = UINT_MAX,
+	},
+
+	/* HTTP requests parameters */
+	{
+		.id = 9000,
+		.name = "http.user-agent",
+		.type = &gt_string,
+		.offset = offsetof(struct rpki_config, http.user_agent),
+		.doc = "User-Agent to use at HTTP requests, eg. Fort Validator Local/1.0",
+	},
+	{
+		.id = 9001,
+		.name = "http.connect-timeout",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, http.connect_timeout),
+		.doc = "Timeout for the connect phase",
+		.min = 1,
+		.max = UINT_MAX,
+	},
+	{
+		.id = 9002,
+		.name = "http.transfer-timeout",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, http.transfer_timeout),
+		.doc = "Maximum transfer time (once the connection is established) before dropping the connection",
+		.min = 0,
+		.max = UINT_MAX,
+	},
+	{
+		.id = 9003,
+		.name = "http.idle-timeout",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, http.idle_timeout),
+		.doc = "Maximum idle time (once the connection is established) during a request before dropping the connection",
+		.min = 0,
+		.max = UINT_MAX,
+	},
+	{
+		.id = 9004,
+		.name = "http.ca-path",
+		.type = &gt_string,
+		.offset = offsetof(struct rpki_config, http.ca_path),
+		.doc = "Directory where CA certificates are found, used to verify the peer",
+		.arg_doc = "<directory>",
+	},
+
 	/* Logging fields */
 	{
 		.id = 'c',
@@ -376,6 +551,16 @@ static const struct option_field options[] = {
 		.arg_doc = "<file>",
 	},
 
+	{
+		.id = 8000,
+		.name = "asn1-decode-max-stack",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, asn1_decode_max_stack),
+		.doc = "ASN1 decoder max stack size, utilized to avoid a stack overflow on large nested ASN1 objects",
+		.min = 1,
+		.max = UINT_MAX,
+	},
+
 	{ 0 },
 };
 
@@ -511,6 +696,7 @@ set_default_values(void)
 		"--delete",
 		"--times",
 		"--contimeout=20",
+		"--timeout=15",
 		"$REMOTE",
 		"$LOCAL",
 	};
@@ -518,6 +704,7 @@ set_default_values(void)
 	static char const *flat_rsync_args[] = {
 		"--times",
 		"--contimeout=20",
+		"--timeout=15",
 		"--dirs",
 		"$REMOTE",
 		"$LOCAL",
@@ -550,11 +737,17 @@ set_default_values(void)
 		goto revert_port;
 	}
 
-	rpki_config.sync_strategy = SYNC_ROOT;
+	rpki_config.sync_strategy = RSYNC_ROOT_EXCEPT_TA;
 	rpki_config.shuffle_tal_uris = false;
 	rpki_config.maximum_certificate_depth = 32;
 	rpki_config.mode = SERVER;
+	rpki_config.work_offline = false;
 
+	rpki_config.rsync.enabled = true;
+	rpki_config.rsync.priority = 50;
+	rpki_config.rsync.strategy = RSYNC_ROOT_EXCEPT_TA;
+	rpki_config.rsync.retry.count = 2;
+	rpki_config.rsync.retry.interval = 5;
 	rpki_config.rsync.program = strdup("rsync");
 	if (rpki_config.rsync.program == NULL) {
 		error = pr_enomem();
@@ -571,6 +764,21 @@ set_default_values(void)
 	if (error)
 		goto revert_recursive_array;
 
+	rpki_config.rrdp.enabled = true;
+	rpki_config.rrdp.priority = 50;
+	rpki_config.rrdp.retry.count = 2;
+	rpki_config.rrdp.retry.interval = 5;
+
+	rpki_config.http.user_agent = strdup(PACKAGE_NAME "/" PACKAGE_VERSION);
+	if (rpki_config.http.user_agent == NULL) {
+		error = pr_enomem();
+		goto revert_flat_array;
+	}
+	rpki_config.http.connect_timeout = 30;
+	rpki_config.http.transfer_timeout = 0;
+	rpki_config.http.idle_timeout = 15;
+	rpki_config.http.ca_path = NULL; /* Use system default */
+
 	rpki_config.log.color = false;
 	rpki_config.log.filename_format = FNF_GLOBAL;
 	rpki_config.log.level = LOG_WARNING;
@@ -579,8 +787,11 @@ set_default_values(void)
 	rpki_config.output.roa = NULL;
 	rpki_config.output.bgpsec = NULL;
 
-	return 0;
+	rpki_config.asn1_decode_max_stack = 4096; /* 4kB */
 
+	return 0;
+revert_flat_array:
+	string_array_cleanup(&rpki_config.rsync.args.flat);
 revert_recursive_array:
 	string_array_cleanup(&rpki_config.rsync.args.recursive);
 revert_rsync_program:
@@ -601,6 +812,12 @@ valid_output_file(char const *path)
 static int
 validate_config(void)
 {
+	if (rpki_config.tal == NULL)
+		return pr_err("The TAL file/directory (--tal) is mandatory.");
+
+	if (!valid_file_or_dir(rpki_config.tal, true, true))
+		return pr_err("Invalid TAL file/directory.");
+
 	if (rpki_config.server.interval.expire <
 	    rpki_config.server.interval.refresh ||
 	    rpki_config.server.interval.expire <
@@ -613,11 +830,17 @@ validate_config(void)
 
 	if (rpki_config.output.bgpsec != NULL &&
 	    !valid_output_file(rpki_config.output.bgpsec))
-			return pr_err("Invalid output.bgpsec file.");
+		return pr_err("Invalid output.bgpsec file.");
 
-	return (rpki_config.tal != NULL)
-	    ? 0
-	    : pr_err("The TAL file/directory (--tal) is mandatory.");
+	if (rpki_config.slurm != NULL &&
+	    !valid_file_or_dir(rpki_config.slurm, true, true))
+		return pr_err("Invalid slurm location.");
+
+	/* FIXME (later) Remove when sync-strategy is fully deprecated */
+	if (!rpki_config.rsync.enabled)
+		config_set_sync_strategy(RSYNC_OFF);
+
+	return 0;
 }
 
 static void
@@ -757,6 +980,12 @@ config_get_server_queue(void)
 	return rpki_config.server.backlog;
 }
 
+bool
+config_get_work_offline(void)
+{
+	return rpki_config.work_offline;
+}
+
 unsigned int
 config_get_validation_interval(void)
 {
@@ -799,12 +1028,6 @@ config_get_local_repository(void)
 	return rpki_config.local_repository;
 }
 
-enum sync_strategy
-config_get_sync_strategy(void)
-{
-	return rpki_config.sync_strategy;
-}
-
 bool
 config_get_shuffle_tal_uris(void)
 {
@@ -841,6 +1064,36 @@ config_get_log_output(void)
 	return rpki_config.log.output;
 }
 
+bool
+config_get_rsync_enabled(void)
+{
+	return rpki_config.rsync.enabled;
+}
+
+unsigned int
+config_get_rsync_priority(void)
+{
+	return rpki_config.rsync.priority;
+}
+
+enum rsync_strategy
+config_get_rsync_strategy(void)
+{
+	return rpki_config.rsync.strategy;
+}
+
+unsigned int
+config_get_rsync_retry_count(void)
+{
+	return rpki_config.rsync.retry.count;
+}
+
+unsigned int
+config_get_rsync_retry_interval(void)
+{
+	return rpki_config.rsync.retry.interval;
+}
+
 char *
 config_get_rsync_program(void)
 {
@@ -850,20 +1103,74 @@ config_get_rsync_program(void)
 struct string_array const *
 config_get_rsync_args(bool is_ta)
 {
-	switch (rpki_config.sync_strategy) {
-	case SYNC_ROOT:
+	switch (rpki_config.rsync.strategy) {
+	case RSYNC_ROOT:
 		return &rpki_config.rsync.args.recursive;
-	case SYNC_ROOT_EXCEPT_TA:
+	case RSYNC_ROOT_EXCEPT_TA:
 		return is_ta
 		    ? &rpki_config.rsync.args.flat
 		    : &rpki_config.rsync.args.recursive;
-	case SYNC_STRICT:
+	case RSYNC_STRICT:
 		return &rpki_config.rsync.args.flat;
-	case SYNC_OFF:
+	default:
 		break;
 	}
 
-	pr_crit("Invalid sync strategy: '%u'", rpki_config.sync_strategy);
+	pr_crit("Invalid rsync strategy: '%u'", rpki_config.rsync.strategy);
+}
+
+bool
+config_get_rrdp_enabled(void)
+{
+	return rpki_config.rrdp.enabled;
+}
+
+unsigned int
+config_get_rrdp_priority(void)
+{
+	return rpki_config.rrdp.priority;
+}
+
+unsigned int
+config_get_rrdp_retry_count(void)
+{
+	return rpki_config.rrdp.retry.count;
+}
+
+unsigned int
+config_get_rrdp_retry_interval(void)
+{
+	return rpki_config.rrdp.retry.interval;
+}
+
+char const *
+config_get_http_user_agent(void)
+{
+	return rpki_config.http.user_agent;
+}
+
+unsigned int
+config_get_http_connect_timeout(void)
+{
+	return rpki_config.http.connect_timeout;
+}
+
+unsigned int
+config_get_http_transfer_timeout(void)
+{
+	return rpki_config.http.transfer_timeout;
+}
+
+unsigned int
+config_get_http_idle_timeout(void)
+{
+	return rpki_config.http.idle_timeout;
+}
+
+char const *
+config_get_http_ca_path(void)
+{
+	return rpki_config.http.ca_path;
 }
 
 char const *
@@ -878,6 +1185,36 @@ config_get_output_bgpsec(void)
 	return rpki_config.output.bgpsec;
 }
 
+unsigned int
+config_get_asn1_decode_max_stack(void)
+{
+	return rpki_config.asn1_decode_max_stack;
+}
+
+void
+config_set_rsync_enabled(bool value)
+{
+	rpki_config.rsync.enabled = value;
+}
+
+void
+config_set_rrdp_enabled(bool value)
+{
+	rpki_config.rrdp.enabled = value;
+}
+
+void
+config_set_sync_strategy(enum rsync_strategy value)
+{
+	rpki_config.sync_strategy = value;
+}
+
+void
+config_set_rsync_strategy(enum rsync_strategy value)
+{
+	rpki_config.rsync.strategy = value;
+}
+
 void
 free_rpki_config(void)
 {
