@@ -6,8 +6,10 @@
 #include "uri.h"
 #include "asn1/asn1c/BIT_STRING.h"
 
-int hash_validate_file(char const *, struct rpki_uri *uri,
+int hash_validate_mft_file(char const *, struct rpki_uri *uri,
     BIT_STRING_t const *);
+int hash_validate_file(char const *, struct rpki_uri *, unsigned char const *,
+    size_t);
 int hash_validate(char const *, unsigned char const *, size_t,
     unsigned char const *, size_t);
 int hash_validate_octet_string(char const *, OCTET_STRING_t const*,
