@@ -4,8 +4,11 @@
 #include "extension.h"
 #include "nid.h"
 #include "thread_var.h"
+#include "http/http.h"
 #include "rtr/rtr.h"
 #include "rtr/db/vrps.h"
+#include "xml/relax_ng.h"
+#include "rrdp/db/db_rrdp.h"
 
 static int
 start_rtr_server(void)
@@ -16,8 +19,14 @@ start_rtr_server(void)
 	if (error)
 		goto just_quit;
 
+	error = db_rrdp_init();
+	if (error)
+		goto vrps_cleanup;
+
 	error = rtr_listen();
 
+	db_rrdp_cleanup();
+vrps_cleanup:
 	vrps_destroy();
 just_quit:
 	return error;
@@ -48,8 +57,19 @@ __main(int argc, char **argv)
 	if (error)
 		goto revert_nid;
 
+	error = http_init();
+	if (error)
+		goto revert_nid;
+
+	error = relax_ng_init();
+	if (error)
+		goto revert_http;
+
 	error = start_rtr_server();
 
+	relax_ng_cleanup();
+revert_http:
+	http_cleanup();
 revert_nid:
 	nid_destroy();
 revert_config:
