@@ -18,6 +18,8 @@
 #include "object/bgpsec.h"
 #include "object/name.h"
 #include "object/manifest.h"
+#include "object/signed_object.h"
+#include "rrdp/rrdp_loader.h"
 #include "rsync/rsync.h"
 
 /* Just to prevent some line breaking. */
@@ -35,9 +37,15 @@ struct ski_arguments {
 	OCTET_STRING_t *sid;
 };
 
-struct sia_uris {
-	struct rpki_uri **caRepository;
-	struct rpki_uri **mft;
+struct sia_uri {
+	uint8_t position;
+	struct rpki_uri *uri;
+};
+
+struct sia_ca_uris {
+	struct sia_uri caRepository;
+	struct sia_uri rpkiNotify;
+	struct sia_uri mft;
 };
 
 struct bgpsec_ski {
@@ -45,6 +53,28 @@ struct bgpsec_ski {
 	unsigned char **ski_data;
 };
 
+/* Callback method to fetch repository objects */
+typedef int (access_method_exec)(struct sia_ca_uris *);
+
+static void
+sia_ca_uris_init(struct sia_ca_uris *sia_uris)
+{
+	sia_uris->caRepository.uri = NULL;
+	sia_uris->rpkiNotify.uri = NULL;
+	sia_uris->mft.uri = NULL;
+}
+
+static void
+sia_ca_uris_cleanup(struct sia_ca_uris *sia_uris)
+{
+	if (sia_uris->caRepository.uri != NULL)
+		uri_refput(sia_uris->caRepository.uri);
+	if (sia_uris->rpkiNotify.uri != NULL)
+		uri_refput(sia_uris->rpkiNotify.uri);
+	if (sia_uris->mft.uri != NULL)
+		uri_refput(sia_uris->mft.uri);
+}
+
 static void
 debug_serial_number(BIGNUM *number)
 {
@@ -115,30 +145,13 @@ validate_issuer(X509 *cert, bool is_ta)
 	return 0;
 }
 
+/*
+ * Compare public keys, call @diff_alg_cb when the algorithm is different, call
+ * @diff_pk_cb when the public key is different; return 0 if both are equal.
+ */
 static int
-validate_subject(X509 *cert)
-{
-	struct validation *state;
-	struct rfc5280_name *name;
-	int error;
-
-	state = state_retrieve();
-	if (state == NULL)
-		return -EINVAL;
-
-	error = x509_name_decode(X509_get_subject_name(cert), "subject", &name);
-	if (error)
-		return error;
-	pr_debug("Subject: %s", x509_name_commonName(name));
-
-	error = x509stack_store_subject(validation_certstack(state), name);
-
-	x509_name_put(name);
-	return error;
-}
-
-static int
-spki_cmp(X509_PUBKEY *tal_spki, X509_PUBKEY *cert_spki)
+spki_cmp(X509_PUBKEY *tal_spki, X509_PUBKEY *cert_spki,
+    int (*diff_alg_cb)(void), int (*diff_pk_cb)(void))
 {
 	ASN1_OBJECT *tal_alg;
 	ASN1_OBJECT *cert_alg;
@@ -158,17 +171,167 @@ spki_cmp(X509_PUBKEY *tal_spki, X509_PUBKEY *cert_spki)
 		return crypto_err("X509_PUBKEY_get0_param() 2 returned %d", ok);
 
 	if (OBJ_cmp(tal_alg, cert_alg) != 0)
-		goto different_alg;
+		return diff_alg_cb();
 	if (tal_spk_len != cert_spk_len)
-		goto different_pk;
+		return diff_pk_cb();
 	if (memcmp(tal_spk, cert_spk, cert_spk_len) != 0)
-		goto different_pk;
+		return diff_pk_cb();
 
 	return 0;
+}
 
-different_alg:
+/** Call whenever the public key is different and the subject is the same**/
+static int
+cert_different_pk_err(void)
+{
+	return 1;
+}
+
+/*
+ * Beware: this function skips a lot (maybe all) RPKI validations for a signed
+ * object, and clearly expects that the signed object has at least one
+ * certificate.
+ *
+ * Allocates @result, don't forget to release it.
+ *
+ * It has been placed here to minimize the risk of misuse.
+ */
+static int
+cert_from_signed_object(struct rpki_uri *uri, uint8_t **result, int *size)
+{
+	struct signed_object sobj;
+	ANY_t *cert;
+	unsigned char *tmp;
+	int error;
+
+	error = signed_object_decode(&sobj, uri);
+	if (error)
+		return error;
+
+	cert = sobj.sdata.decoded->certificates->list.array[0];
+
+	tmp = malloc(cert->size);
+	if (tmp == NULL) {
+		signed_object_cleanup(&sobj);
+		return pr_enomem();
+	}
+
+	memcpy(tmp, cert->buf, cert->size);
+
+	*result = tmp;
+	(*size) = cert->size;
+
+	signed_object_cleanup(&sobj);
+	return 0;
+}
+
+static int
+check_dup_public_key(bool *duplicated, char const *file, void *arg)
+{
+	X509 *curr_cert = arg; /* Current cert */
+	X509 *rcvd_cert;
+	X509_PUBKEY *curr_pk, *rcvd_pk;
+	struct rpki_uri *uri;
+	uint8_t *tmp;
+	int tmp_size;
+	int error;
+
+	uri = NULL;
+	rcvd_cert = NULL;
+	tmp_size = 0;
+
+	error = uri_create_rsync_str(&uri, file, strlen(file));
+	if (error)
+		return error;
+
+	/* Check if it's '.cer', otherwise treat as a signed object */
+	if (uri_is_certificate(uri)) {
+		error = certificate_load(uri, &rcvd_cert);
+		if (error)
+			goto free_uri;
+	} else {
+		error = cert_from_signed_object(uri, &tmp, &tmp_size);
+		if (error)
+			goto free_uri;
+
+		rcvd_cert = d2i_X509(NULL, (const unsigned char **) &tmp,
+		    tmp_size);
+		free(tmp); /* Release at once */
+		if (rcvd_cert == NULL) {
+			error = crypto_err("Signed object's '%s' 'certificate' element does not decode into a Certificate",
+			    uri_get_printable(uri));
+			goto free_uri;
+		}
+	}
+
+	curr_pk = X509_get_X509_PUBKEY(curr_cert);
+	if (curr_pk == NULL) {
+		error = crypto_err("X509_get_X509_PUBKEY() returned NULL");
+		goto free_cert;
+	}
+
+	rcvd_pk = X509_get_X509_PUBKEY(rcvd_cert);
+	if (rcvd_pk == NULL) {
+		error = crypto_err("X509_get_X509_PUBKEY() returned NULL");
+		goto free_cert;
+	}
+
+	/*
+	 * The function response will be:
+	 *   < 0 if there's an error
+	 *   = 0 if the PKs are equal
+	 *   > 0 if the PKs are different
+	 */
+	error = spki_cmp(curr_pk, rcvd_pk, cert_different_pk_err,
+	    cert_different_pk_err);
+
+	if (error < 0)
+		goto free_cert;
+
+	/* No error, a positive value means the name is duplicated */
+	if (error)
+		(*duplicated) = true;
+	error = 0;
+
+free_cert:
+	X509_free(rcvd_cert);
+free_uri:
+	uri_refput(uri);
+	return error;
+}
+
+static int
+validate_subject(X509 *cert)
+{
+	struct validation *state;
+	struct rfc5280_name *name;
+	int error;
+
+	state = state_retrieve();
+	if (state == NULL)
+		return -EINVAL;
+
+	error = x509_name_decode(X509_get_subject_name(cert), "subject", &name);
+	if (error)
+		return error;
+	pr_debug("Subject: %s", x509_name_commonName(name));
+
+	error = x509stack_store_subject(validation_certstack(state), name,
+	    check_dup_public_key, cert);
+
+	x509_name_put(name);
+	return error;
+}
+
+static int
+root_different_alg_err(void)
+{
 	return pr_err("TAL's public key algorithm is different than the root certificate's public key algorithm.");
-different_pk:
+}
+
+static int
+root_different_pk_err(void)
+{
 	return pr_err("TAL's public key is different than the root certificate's public key.");
 }
 
@@ -193,7 +356,7 @@ validate_spki(X509_PUBKEY *cert_spki)
 	/*
 	 * We have a problem at this point:
 	 *
-	 * RFC 7730 says "The public key used to verify the trust anchor MUST be
+	 * RFC 8630 says "The public key used to verify the trust anchor MUST be
 	 * the same as the subjectPublicKeyInfo in the CA certificate and in the
 	 * TAL."
 	 *
@@ -218,7 +381,8 @@ validate_spki(X509_PUBKEY *cert_spki)
 		goto fail1;
 	}
 
-	if (spki_cmp(tal_spki, cert_spki) != 0)
+	if (spki_cmp(tal_spki, cert_spki, root_different_alg_err,
+	    root_different_pk_err) != 0)
 		goto fail2;
 
 	X509_PUBKEY_free(tal_spki);
@@ -380,7 +544,7 @@ certificate_validate_rfc6487(X509 *cert, enum cert_type type)
 	/* libcrypto already does this. */
 
 	/* rfc6487#section-4.7 */
-	/* Fragment of rfc7730#section-2.2 */
+	/* Fragment of rfc8630#section-2.3 */
 	error = validate_public_key(cert, type);
 	if (error)
 		return error;
@@ -740,7 +904,7 @@ handle_ip_extension(X509_EXTENSION *ext, struct resources *resources)
 
 	string = X509_EXTENSION_get_data(ext);
 	error = asn1_decode(string->data, string->length, &asn_DEF_IPAddrBlocks,
-	    (void **) &blocks, true);
+	    (void **) &blocks, true, false);
 	if (error)
 		return error;
 
@@ -788,7 +952,7 @@ handle_asn_extension(X509_EXTENSION *ext, struct resources *resources,
 
 	string = X509_EXTENSION_get_data(ext);
 	error = asn1_decode(string->data, string->length,
-	    &asn_DEF_ASIdentifiers, (void **) &ids, true);
+	    &asn_DEF_ASIdentifiers, (void **) &ids, true, false);
 	if (error)
 		return error;
 
@@ -902,26 +1066,39 @@ is_rsync_uri(GENERAL_NAME *name)
 }
 
 static int
-handle_rpkiManifest(struct rpki_uri *uri, void *arg)
+handle_rpkiManifest(struct rpki_uri *uri, uint8_t pos, void *arg)
 {
-	struct rpki_uri **mft = arg;
-	*mft = uri;
+	struct sia_uri *mft = arg;
+	mft->position = pos;
+	mft->uri = uri;
 	uri_refget(uri);
 	return 0;
 }
 
 static int
-handle_caRepository(struct rpki_uri *uri, void *arg)
+handle_caRepository(struct rpki_uri *uri, uint8_t pos, void *arg)
 {
-	struct rpki_uri **repo = arg;
+	struct sia_uri *repo = arg;
 	pr_debug("caRepository: %s", uri_get_printable(uri));
-	*repo = uri;
+	repo->position = pos;
+	repo->uri = uri;
 	uri_refget(uri);
-	return download_files(uri, false, false);
+	return 0;
 }
 
 static int
-handle_signedObject(struct rpki_uri *uri, void *arg)
+handle_rpkiNotify(struct rpki_uri *uri, uint8_t pos, void *arg)
+{
+	struct sia_uri *notify = arg;
+	pr_debug("rpkiNotify: %s", uri_get_printable(uri));
+	notify->position = pos;
+	notify->uri = uri;
+	uri_refget(uri);
+	return 0;
+}
+
+static int
+handle_signedObject(struct rpki_uri *uri, uint8_t pos, void *arg)
 {
 	struct certificate_refs *refs = arg;
 	pr_debug("signedObject: %s", uri_get_printable(uri));
@@ -1230,31 +1407,42 @@ end:
  */
 static int
 handle_ad(char const *ia_name, SIGNATURE_INFO_ACCESS *ia,
-    char const *ad_name, int ad_nid,
-    int (*cb)(struct rpki_uri *, void *), void *arg)
+    char const *ad_name, int ad_nid, int uri_flags, bool required,
+    int (*cb)(struct rpki_uri *, uint8_t, void *), void *arg)
 {
+# define AD_METHOD (uri_flags == URI_VALID_RSYNC ? \
+	"RSYNC" : \
+	((uri_flags == URI_VALID_HTTPS) ? "HTTPS" : "RSYNC/HTTPS"))
 	ACCESS_DESCRIPTION *ad;
 	struct rpki_uri *uri;
 	bool found = false;
-	int i;
+	unsigned int i;
 	int error;
 
 	for (i = 0; i < sk_ACCESS_DESCRIPTION_num(ia); i++) {
 		ad = sk_ACCESS_DESCRIPTION_value(ia, i);
 		if (OBJ_obj2nid(ad->method) == ad_nid) {
-			error = uri_create_ad(&uri, ad);
-			if (error == ENOTRSYNC)
+			error = uri_create_ad(&uri, ad, uri_flags);
+			switch (error) {
+			case 0:
+				break;
+			case ENOTRSYNC:
 				continue;
-			if (error)
+			case ENOTHTTPS:
+				continue;
+			case ENOTSUPPORTED:
+				continue;
+			default:
 				return error;
+			}
 
 			if (found) {
 				uri_refput(uri);
-				return pr_err("Extension '%s' has multiple '%s' RSYNC URIs.",
-				    ia_name, ad_name);
+				return pr_err("Extension '%s' has multiple '%s' %s URIs.",
+				    ia_name, ad_name, AD_METHOD);
 			}
 
-			error = cb(uri, arg);
+			error = cb(uri, i, arg);
 			if (error) {
 				uri_refput(uri);
 				return error;
@@ -1265,9 +1453,9 @@ handle_ad(char const *ia_name, SIGNATURE_INFO_ACCESS *ia,
 		}
 	}
 
-	if (!found) {
-		pr_err("Extension '%s' lacks a '%s' RSYNC URI.", ia_name,
-		    ad_name);
+	if (required && !found) {
+		pr_err("Extension '%s' lacks a '%s' valid %s URI.", ia_name,
+		    ad_name, AD_METHOD);
 		return -ESRCH;
 	}
 
@@ -1275,7 +1463,7 @@ handle_ad(char const *ia_name, SIGNATURE_INFO_ACCESS *ia,
 }
 
 static int
-handle_caIssuers(struct rpki_uri *uri, void *arg)
+handle_caIssuers(struct rpki_uri *uri, uint8_t pos, void *arg)
 {
 	struct certificate_refs *refs = arg;
 	/*
@@ -1299,7 +1487,7 @@ handle_aia(X509_EXTENSION *ext, void *arg)
 		return cannot_decode(ext_aia());
 
 	error = handle_ad("AIA", aia, "caIssuers", NID_ad_ca_issuers,
-	    handle_caIssuers, arg);
+	    URI_VALID_RSYNC, true, handle_caIssuers, arg);
 
 	AUTHORITY_INFO_ACCESS_free(aia);
 	return error;
@@ -1309,16 +1497,22 @@ static int
 handle_sia_ca(X509_EXTENSION *ext, void *arg)
 {
 	SIGNATURE_INFO_ACCESS *sia;
-	struct sia_uris *uris = arg;
+	struct sia_ca_uris *uris = arg;
 	int error;
 
 	sia = X509V3_EXT_d2i(ext);
 	if (sia == NULL)
 		return cannot_decode(ext_sia());
 
-	/* rsync */
+	/* rsync, still the preferred and required */
 	error = handle_ad("SIA", sia, "caRepository", NID_caRepository,
-	    handle_caRepository, uris->caRepository);
+	    URI_VALID_RSYNC, true, handle_caRepository, &uris->caRepository);
+	if (error)
+		goto end;
+
+	/* HTTPS RRDP */
+	error = handle_ad("SIA", sia, "rpkiNotify", nid_rpkiNotify(),
+	    URI_VALID_HTTPS, false, handle_rpkiNotify, &uris->rpkiNotify);
 	if (error)
 		goto end;
 
@@ -1328,7 +1522,7 @@ handle_sia_ca(X509_EXTENSION *ext, void *arg)
 	 * is fully valid.)
 	 */
 	error = handle_ad("SIA", sia, "rpkiManifest", nid_rpkiManifest(),
-	    handle_rpkiManifest, uris->mft);
+	    URI_VALID_RSYNC, true, handle_rpkiManifest, &uris->mft);
 
 end:
 	AUTHORITY_INFO_ACCESS_free(sia);
@@ -1346,7 +1540,7 @@ handle_sia_ee(X509_EXTENSION *ext, void *arg)
 		return cannot_decode(ext_sia());
 
 	error = handle_ad("SIA", sia, "signedObject", nid_signedObject(),
-	    handle_signedObject, arg);
+	    URI_VALID_RSYNC, true, handle_signedObject, arg);
 
 	AUTHORITY_INFO_ACCESS_free(sia);
 	return error;
@@ -1457,21 +1651,20 @@ end:
 /**
  * Validates the certificate extensions, Trust Anchor style.
  *
- * Also initializes the second argument as the URI of the rpkiManifest Access
- * Description and the third arg as the CA Repository from the SIA extension.
+ * Depending on the Access Description preference at SIA, the rpki_uri's at
+ * @sia_uris will be allocated.
  */
 static int
-certificate_validate_extensions_ta(X509 *cert, struct rpki_uri **mft,
-    struct rpki_uri **caRepository, enum rpki_policy *policy)
+certificate_validate_extensions_ta(X509 *cert, struct sia_ca_uris *sia_uris,
+    enum rpki_policy *policy)
 {
-	struct sia_uris sia_uris;
 	struct extension_handler handlers[] = {
 	   /* ext        reqd   handler        arg       */
 	    { ext_bc(),  true,  handle_bc,               },
 	    { ext_ski(), true,  handle_ski_ca, cert      },
 	    { ext_aki(), false, handle_aki_ta, cert      },
 	    { ext_ku(),  true,  handle_ku_ca,            },
-	    { ext_sia(), true,  handle_sia_ca, &sia_uris },
+	    { ext_sia(), true,  handle_sia_ca, sia_uris  },
 	    { ext_cp(),  true,  handle_cp,     policy    },
 	    { ext_ir(),  false, handle_ir,               },
 	    { ext_ar(),  false, handle_ar,               },
@@ -1480,9 +1673,6 @@ certificate_validate_extensions_ta(X509 *cert, struct rpki_uri **mft,
 	    { NULL },
 	};
 
-	sia_uris.caRepository = caRepository;
-	sia_uris.mft = mft;
-
 	return handle_extensions(handlers, X509_get0_extensions(cert));
 }
 
@@ -1490,17 +1680,15 @@ certificate_validate_extensions_ta(X509 *cert, struct rpki_uri **mft,
  * Validates the certificate extensions, (intermediate) Certificate Authority
  * style.
  *
- * Also initializes the second argument as the URI of the rpkiManifest Access
- * Description and the third arg as the CA Repository from the SIA extension.
+ * Depending on the Access Description preference at SIA, the rpki_uri's at
+ * @sia_uris will be allocated.
  * Also initializes the fourth argument with the references found in the
  * extensions.
  */
 static int
-certificate_validate_extensions_ca(X509 *cert, struct rpki_uri **mft,
-    struct rpki_uri **caRepository, struct certificate_refs *refs,
-    enum rpki_policy *policy)
+certificate_validate_extensions_ca(X509 *cert, struct sia_ca_uris *sia_uris,
+    struct certificate_refs *refs, enum rpki_policy *policy)
 {
-	struct sia_uris sia_uris;
 	struct extension_handler handlers[] = {
 	   /* ext        reqd   handler        arg       */
 	    { ext_bc(),  true,  handle_bc,               },
@@ -1509,7 +1697,7 @@ certificate_validate_extensions_ca(X509 *cert, struct rpki_uri **mft,
 	    { ext_ku(),  true,  handle_ku_ca,            },
 	    { ext_cdp(), true,  handle_cdp,    refs      },
 	    { ext_aia(), true,  handle_aia,    refs      },
-	    { ext_sia(), true,  handle_sia_ca, &sia_uris },
+	    { ext_sia(), true,  handle_sia_ca, sia_uris  },
 	    { ext_cp(),  true,  handle_cp,     policy    },
 	    { ext_ir(),  false, handle_ir,               },
 	    { ext_ar(),  false, handle_ar,               },
@@ -1518,9 +1706,6 @@ certificate_validate_extensions_ca(X509 *cert, struct rpki_uri **mft,
 	    { NULL },
 	};
 
-	sia_uris.caRepository = caRepository;
-	sia_uris.mft = mft;
-
 	return handle_extensions(handlers, X509_get0_extensions(cert));
 }
 
@@ -1588,6 +1773,236 @@ get_certificate_type(X509 *cert, bool is_ta)
 	return EE;
 }
 
+/*
+ * It does some of the things from validate_issuer(), but we can not wait for
+ * such validation, since at this point the RSYNC URI at AIA extension must be
+ * verified to comply with rfc6487#section-4.8.7
+ */
+static int
+force_aia_validation(struct rpki_uri *caIssuers, void *arg)
+{
+	X509 *son = arg;
+	X509 *parent;
+	struct rfc5280_name *son_name;
+	struct rfc5280_name *parent_name;
+	int error;
+
+	pr_debug("AIA's URI didn't matched parent URI, doing AIA RSYNC");
+
+	/* RSYNC is still the prefered access mechanism */
+	error = download_files(caIssuers, false, false);
+	if (error)
+		return error;
+
+	error = certificate_load(caIssuers, &parent);
+	if (error)
+		return error;
+
+	error = x509_name_decode(X509_get_subject_name(parent), "subject",
+	    &parent_name);
+	if (error)
+		goto free_parent;
+
+	error = x509_name_decode(X509_get_issuer_name(son), "issuer",
+	    &son_name);
+	if (error)
+		goto free_parent_name;
+
+	if (x509_name_equals(parent_name, son_name))
+		error = 0; /* Everything its ok */
+	else
+		error = pr_err("Certificate subject from AIA ('%s') isn't issuer of this certificate.",
+		    uri_get_printable(caIssuers));
+
+	x509_name_put(son_name);
+free_parent_name:
+	x509_name_put(parent_name);
+free_parent:
+	X509_free(parent);
+	return error;
+}
+
+int
+certificate_validate_aia(struct rpki_uri *caIssuers, X509 *cert)
+{
+	struct validation *state;
+	struct rpki_uri *parent;
+
+	if (caIssuers == NULL)
+		pr_crit("Certificate's AIA was not recorded.");
+
+	state = state_retrieve();
+	if (state == NULL)
+		return -EINVAL;
+	parent = x509stack_peek_uri(validation_certstack(state));
+	if (parent == NULL)
+		pr_crit("Certificate has no parent.");
+
+	/*
+	 * There are two possible issues here, specifically at first level root
+	 * certificate's childs:
+	 *
+	 * - Considering that the root certificate can be published at one or
+	 *   more rsync or HTTPS URIs (RFC 8630), the validation is done
+	 *   considering the first valid downloaded certificate URI from the
+	 *   list of URIs; so, that URI doesn't necessarily matches AIA. And
+	 *   this issue is more likely to happen if the 'shuffle-uris' flag
+	 *   is active an a TAL has more than one rsync/HTTPS uri.
+	 *
+	 * - If the TAL has only one URI, and such URI is HTTPS, the root
+	 *   certificate will be located at a distinct point that what it's
+	 *   expected, so this might be an error if such certificate (root
+	 *   certificate) isn't published at an rsync repository. See RFC 6487
+	 *   section-4.8.7:
+	 *
+	 *   "The preferred URI access mechanisms is "rsync", and an rsync URI
+	 *   [RFC5781] MUST be specified with an accessMethod value of
+	 *   id-ad-caIssuers.  The URI MUST reference the point of publication
+	 *   of the certificate where this Issuer is the subject (the issuer's
+	 *   immediate superior certificate)."
+	 *
+	 * January 2020: this isn't a problem, all five RIRs are using one URI
+	 * at their TALs, that matches AIA from the first level root certificate
+	 * childs. Anyways, we'll try to consult the subject at IETF.
+	 */
+	if (uri_equals(caIssuers, parent))
+		return 0;
+
+	/*
+	 * URI didn't match, try to match the immediate superior subject with
+	 * the current issuer. This will force an RSYNC of AIA's URI, load
+	 * the certificate and do the comparison.
+	 */
+	return force_aia_validation(caIssuers, cert);
+}
+
+/*
+ * Verify that the manifest file actually exists at the local repository, if it
+ * doesn't exist then discard the repository (which can result in a attempt
+ * to fetch data from another repository).
+ */
+static int
+verify_mft_loc(struct rpki_uri *mft_uri)
+{
+	if (!valid_file_or_dir(uri_get_local(mft_uri), true, false))
+		return -EINVAL; /* Error already logged */
+
+	return 0;
+}
+
+static int
+exec_rrdp_method(struct sia_ca_uris *sia_uris)
+{
+	int error;
+
+	error = rrdp_load(sia_uris->rpkiNotify.uri);
+	if (error)
+		return error;
+
+	return verify_mft_loc(sia_uris->mft.uri);
+}
+
+static int
+exec_rsync_method(struct sia_ca_uris *sia_uris)
+{
+	int error;
+
+	error = download_files(sia_uris->caRepository.uri, false, false);
+	if (error)
+		return error;
+
+	return verify_mft_loc(sia_uris->mft.uri);
+}
+
+/*
+ * Currently only two access methods are supported, just consider those two:
+ * rsync and RRDP. If a new access method is supported, this function must
+ * change (and probably the sia_ca_uris struct as well).
+ */
+static int
+use_access_method(struct sia_ca_uris *sia_uris,
+    access_method_exec rsync_cb, access_method_exec rrdp_cb,
+    bool *rsync_utilized)
+{
+	access_method_exec *cb_primary;
+	access_method_exec *cb_secondary;
+	rrdp_req_status_t rrdp_req_status;
+	bool primary_rrdp;
+	int error;
+
+	/*
+	 * RSYNC will always be present (at least for now, see
+	 * rfc6487#section-4.8.8.1). If rsync is disabled, the cb will take
+	 * care of that.
+	 */
+	(*rsync_utilized) = true;
+	if (sia_uris->rpkiNotify.uri == NULL || !config_get_rrdp_enabled())
+		return rsync_cb(sia_uris);
+
+	/* RSYNC disabled, and RRDP is present, use it */
+	if (!config_get_rsync_enabled()) {
+		(*rsync_utilized) = false;
+		return rrdp_cb(sia_uris);
+	}
+
+	/*
+	 * There isn't any restriction about the preferred access method of
+	 * children CAs being the same as the parent CA.
+	 *
+	 * Two possible scenarios arise:
+	 * 1) CA Parent didn't utilized (or didn't had) and RRDP update
+	 *    notification URI.
+	 * 2) CA Parent successfully utilized an RRDP update notification URI.
+	 *
+	 * Step (1) is simple, do the check of the preferred access method.
+	 * Step (2) must do something different.
+	 * - If RRDP URI was already successfully visited, don't care
+	 *   preference, don't execute access method.
+	 */
+	error = db_rrdp_uris_get_request_status(
+	    uri_get_global(sia_uris->rpkiNotify.uri), &rrdp_req_status);
+	if (error ==  0 && rrdp_req_status == RRDP_URI_REQ_VISITED) {
+		(*rsync_utilized) = false;
+		return 0;
+	}
+
+	/* Use CA's or configured priority? */
+	if (config_get_rsync_priority() == config_get_rrdp_priority())
+		primary_rrdp = sia_uris->caRepository.position
+		    > sia_uris->rpkiNotify.position;
+	else
+		primary_rrdp = config_get_rsync_priority()
+		    < config_get_rrdp_priority();
+
+	cb_primary = primary_rrdp ? rrdp_cb : rsync_cb;
+	cb_secondary = primary_rrdp ? rsync_cb : rrdp_cb;
+
+	/* Try with the preferred; in case of error, try with the next one */
+	error = cb_primary(sia_uris);
+	if (!error) {
+		(*rsync_utilized) = !primary_rrdp;
+		return 0;
+	}
+
+	if (primary_rrdp) {
+		if (error != -EPERM)
+			pr_info("Couldn't fetch data from RRDP repository '%s', trying to fetch data now from '%s'.",
+			    uri_get_global(sia_uris->rpkiNotify.uri),
+			    uri_get_global(sia_uris->caRepository.uri));
+		else
+			pr_info("RRDP repository '%s' download/processing returned error previously, now I will try to fetch data from '%s'.",
+			    uri_get_global(sia_uris->rpkiNotify.uri),
+			    uri_get_global(sia_uris->caRepository.uri));
+	} else {
+		pr_info("Couldn't fetch data from repository '%s', trying to fetch data now from RRDP '%s'.",
+		    uri_get_global(sia_uris->caRepository.uri),
+		    uri_get_global(sia_uris->rpkiNotify.uri));
+	}
+
+	(*rsync_utilized) = primary_rrdp;
+	return cb_secondary(sia_uris);
+}
+
 /** Boilerplate code for CA certificate validation and recursive traversal. */
 int
 certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
@@ -1599,8 +2014,7 @@ certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
 	int total_parents;
 	STACK_OF(X509_CRL) *rpp_parent_crl;
 	X509 *cert;
-	struct rpki_uri *mft;
-	struct rpki_uri *caRepository;
+	struct sia_ca_uris sia_uris;
 	struct certificate_refs refs;
 	unsigned char *ski;
 	enum rpki_policy policy;
@@ -1639,10 +2053,11 @@ certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
 	if (error)
 		goto revert_cert;
 
+	sia_ca_uris_init(&sia_uris);
 	type = get_certificate_type(cert, IS_TA);
 
 	/* Debug cert type */
-	switch(type) {
+	switch (type) {
 	case TA:
 		break;
 	case CA:
@@ -1661,8 +2076,8 @@ certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
 		goto revert_cert;
 	switch (type) {
 	case TA:
-		error = certificate_validate_extensions_ta(cert, &mft,
-		    &caRepository, &policy);
+		error = certificate_validate_extensions_ta(cert, &sia_uris,
+		    &policy);
 		break;
 	case BGPSEC:
 		error = certificate_validate_extensions_bgpsec(cert, &ski,
@@ -1670,13 +2085,19 @@ certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
 		break;
 	default:
 		/* Validate as a CA */
-		error = certificate_validate_extensions_ca(cert, &mft,
-		    &caRepository, &refs, &policy);
+		error = certificate_validate_extensions_ca(cert, &sia_uris,
+		    &refs, &policy);
 		break;
 	}
 	if (error)
 		goto revert_cert;
 
+	if (!IS_TA) {
+		error = certificate_validate_aia(refs.caIssuers, cert);
+		if (error)
+			goto revert_uris;
+	}
+
 	error = refs_validate_ca(&refs, rpp_parent);
 	if (error)
 		goto revert_uris;
@@ -1700,38 +2121,40 @@ certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
 	 *
 	 * Trying to avoid this issue, download the CA repository and validate
 	 * manifest (and its content) again.
+	 * 
+	 * Avoid to re-download the repo if the mft was fetched with RRDP.
 	 */
 	mft_retry = true;
+	error = use_access_method(&sia_uris, exec_rsync_method,
+	    exec_rrdp_method, &mft_retry);
+	if (error)
+		goto revert_uris;
+
 	do {
 		/* Validate the manifest (@mft) pointed by the certificate */
 		error = x509stack_push(validation_certstack(state), cert_uri,
 		    cert, policy, IS_TA);
-		if (error) {
-			if (!mft_retry)
-				uri_refput(mft);
+		if (error)
 			goto revert_uris;
-		}
+
 		cert = NULL; /* Ownership stolen */
 
-		error = handle_manifest(mft, &pp);
-		if (!mft_retry)
-			uri_refput(mft);
-		if (!error || !mft_retry)
+		error = handle_manifest(sia_uris.mft.uri, &pp);
+		if (error == 0 || !mft_retry)
 			break;
 
 		pr_info("Retrying repository download to discard 'transient inconsistency' manifest issue (see RFC 6481 section 5) '%s'",
-		    uri_get_printable(caRepository));
-		error = download_files(caRepository, false, true);
+		    uri_get_printable(sia_uris.caRepository.uri));
+		error = download_files(sia_uris.caRepository.uri, false, true);
 		if (error)
 			break;
 
 		/* Cancel stack, reload certificate (no need to revalidate) */
 		x509stack_cancel(validation_certstack(state));
 		error = certificate_load(cert_uri, &cert);
-		if (error) {
+		if (error)
 			goto revert_uris;
-		}
-		uri_refget(mft);
+
 		mft_retry = false;
 	} while (true);
 
@@ -1745,8 +2168,7 @@ certificate_traverse(struct rpp *rpp_parent, struct rpki_uri *cert_uri)
 
 	rpp_refput(pp);
 revert_uris:
-	uri_refput(caRepository);
-	uri_refput(mft);
+	sia_ca_uris_cleanup(&sia_uris);
 revert_refs:
 	refs_cleanup(&refs);
 revert_cert:
