@@ -8,6 +8,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 
+#include "common.h"
 #include "config.h"
 #include "log.h"
 #include "str.h"
@@ -67,7 +68,7 @@ is_descendant(struct rpki_uri *ancestor, struct rpki_uri *descendant)
 	string_tokenizer_init(&descendant_tokenizer, uri_get_global(descendant),
 	    uri_get_global_len(descendant), '/');
 
-	if (config_get_sync_strategy() == SYNC_STRICT)
+	if (config_get_rsync_strategy() == RSYNC_STRICT)
 		return strcmp(uri_get_global(ancestor),
 		    uri_get_global(descendant)) == 0;
 
@@ -140,7 +141,7 @@ handle_root_strategy(struct rpki_uri *src, struct rpki_uri **dst)
 		if (global[i] == '/') {
 			slashes++;
 			if (slashes == 4)
-				return uri_create_str(dst, global, i);
+				return uri_create_rsync_str(dst, global, i);
 		}
 	}
 
@@ -153,110 +154,44 @@ static int
 get_rsync_uri(struct rpki_uri *requested_uri, bool is_ta,
     struct rpki_uri **rsync_uri)
 {
-	switch (config_get_sync_strategy()) {
-	case SYNC_ROOT:
+	switch (config_get_rsync_strategy()) {
+	case RSYNC_ROOT:
 		return handle_root_strategy(requested_uri, rsync_uri);
-	case SYNC_ROOT_EXCEPT_TA:
+	case RSYNC_ROOT_EXCEPT_TA:
 		return is_ta
 		    ? handle_strict_strategy(requested_uri, rsync_uri)
 		    : handle_root_strategy(requested_uri, rsync_uri);
-	case SYNC_STRICT:
+	case RSYNC_STRICT:
 		return handle_strict_strategy(requested_uri, rsync_uri);
-	case SYNC_OFF:
+	default:
 		break;
 	}
 
-	pr_crit("Invalid sync strategy: %u", config_get_sync_strategy());
+	pr_crit("Invalid rsync strategy: %u", config_get_rsync_strategy());
 }
 
-static int
-dir_exists(char const *path, bool *result)
-{
-	struct stat _stat;
-	char *last_slash;
-
-	last_slash = strrchr(path, '/');
-	if (last_slash == NULL) {
-		/*
-		 * Simply because create_dir_recursive() has nothing meaningful
-		 * to do when this happens. It's a pretty strange error.
-		 */
-		*result = true;
-		return 0;
-	}
-
-	*last_slash = '\0';
-
-	if (stat(path, &_stat) == 0) {
-		if (!S_ISDIR(_stat.st_mode)) {
-			return pr_err("Path '%s' exists and is not a directory.",
-			    path);
-		}
-		*result = true;
-	} else if (errno == ENOENT) {
-		*result = false;
-	} else {
-		return pr_errno(errno, "stat() failed");
-	}
-
-	*last_slash = '/';
-	return 0;
-}
-
-static int
-create_dir(char *path)
-{
-	int error;
-
-	error = mkdir(path, 0777);
-
-	if (error && errno != EEXIST)
-		return pr_errno(errno, "Error while making directory '%s'",
-		    path);
-
-	return 0;
-}
-
-/**
- * Apparently, RSYNC does not like to create parent directories.
- * This function fixes that.
+/*
+ * Duplicate parent FDs, to pipe rsync output:
+ * - fds[0] = stderr
+ * - fds[1] = stdout
  */
-static int
-create_dir_recursive(struct rpki_uri *uri)
+static void
+duplicate_fds(int fds[2][2])
 {
-	char *localuri;
-	int i, error;
-	bool exist = false;
+	/* Use the loop to catch interruptions */
+	while ((dup2(fds[0][1], STDERR_FILENO) == -1)
+		&& (errno == EINTR)) {}
+	close(fds[0][1]);
+	close(fds[0][0]);
 
-	error = dir_exists(uri_get_local(uri), &exist);
-	if (error)
-		return error;
-	if (exist)
-		return 0;
-
-	localuri = strdup(uri_get_local(uri));
-	if (localuri == NULL)
-		return pr_enomem();
-
-	for (i = 1; localuri[i] != '\0'; i++) {
-		if (localuri[i] == '/') {
-			localuri[i] = '\0';
-			error = create_dir(localuri);
-			localuri[i] = '/';
-			if (error) {
-				/* error msg already printed */
-				free(localuri);
-				return error;
-			}
-		}
-	}
-
-	free(localuri);
-	return 0;
+	while ((dup2(fds[1][1], STDOUT_FILENO) == -1)
+	    && (errno == EINTR)) {}
+	close(fds[1][1]);
+	close(fds[1][0]);
 }
 
 static void
-handle_child_thread(struct rpki_uri *uri, bool is_ta)
+handle_child_thread(struct rpki_uri *uri, bool is_ta, int fds[2][2])
 {
 	/* THIS FUNCTION MUST NEVER RETURN!!! */
 
@@ -294,6 +229,8 @@ handle_child_thread(struct rpki_uri *uri, bool is_ta)
 	for (i = 0; i < config_args->length + 1; i++)
 		pr_debug("    %s", copy_args[i]);
 
+	duplicate_fds(fds);
+
 	execvp(copy_args[0], copy_args);
 	error = errno;
 	pr_err("Could not execute the rsync command: %s",
@@ -303,48 +240,157 @@ handle_child_thread(struct rpki_uri *uri, bool is_ta)
 	exit(error);
 }
 
+static int
+create_pipes(int fds[2][2])
+{
+	if (pipe(fds[0]) == -1)
+		return -pr_errno(errno, "Piping rsync stderr");
+	if (pipe(fds[1]) == -1)
+		return -pr_errno(errno, "Piping rsync stdout");
+	return 0;
+}
+
+static void
+log_buffer(char const *buffer, ssize_t read, int type)
+{
+#define PRE_RSYNC "[RSYNC exec]: "
+	char *cpy, *cur, *tmp;
+
+	cpy = malloc(read + 1);
+	if (cpy == NULL) {
+		pr_enomem();
+		return;
+	}
+	strncpy(cpy, buffer, read);
+	cpy[read] = '\0';
+
+	/* Break lines to one line at log */
+	cur = cpy;
+	while ((tmp = strchr(cur, '\n')) != NULL) {
+		*tmp = '\0';
+		if(strlen(cur) == 0) {
+			cur = tmp + 1;
+			continue;
+		}
+		if (type == 0)
+			pr_err(PRE_RSYNC "%s", cur);
+		else
+			pr_info(PRE_RSYNC "%s", cur);
+		cur = tmp + 1;
+	}
+	free(cpy);
+#undef PRE_RSYNC
+}
+
+static int
+read_pipe(int fd_pipe[2][2], int type)
+{
+	char buffer[4096];
+	ssize_t count;
+
+	while (1) {
+		count = read(fd_pipe[type][0], buffer, sizeof(buffer));
+		if (count == -1) {
+			if (errno == EINTR)
+				continue;
+			return -pr_errno(errno, "Reading rsync buffer");
+		}
+		if (count == 0)
+			break;
+
+		log_buffer(buffer, count, type);
+	}
+	close(fd_pipe[type][0]);
+	return 0;
+}
+
+static int
+read_pipes(int fds[2][2])
+{
+	int error;
+
+	/* Won't be needed */
+	close(fds[0][1]);
+	close(fds[1][1]);
+
+	/* stderr pipe */
+	error = read_pipe(fds, 0);
+	if (error)
+		return error;
+
+	/* stdout pipe */
+	return read_pipe(fds, 1);
+}
+
 /*
  * Downloads the @uri->global file into the @uri->local path.
  */
 static int
 do_rsync(struct rpki_uri *uri, bool is_ta)
 {
+	/* Descriptors to pipe stderr (first element) and stdout (second) */
+	int fork_fds[2][2];
 	pid_t child_pid;
+	unsigned int retries;
 	int child_status;
 	int error;
 
-	child_status = 0;
-	error = create_dir_recursive(uri);
-	if (error)
-		return error;
-
-	/* We need to fork because execvp() magics the thread away. */
-	child_pid = fork();
-	if (child_pid == 0) {
-		/* This code is run by the child. */
-		handle_child_thread(uri, is_ta);
-	}
-
-	/* This code is run by us. */
-
-	error = waitpid(child_pid, &child_status, 0);
+	retries = 0;
 	do {
-		if (error == -1) {
-			error = errno;
-			pr_err("The rsync sub-process returned error %d (%s)",
-			    error, strerror(error));
-			if (child_status > 0)
-				break;
+		child_status = 0;
+		error = create_dir_recursive(uri_get_local(uri));
+		if (error)
 			return error;
-		}
-	} while (0);
 
-	if (WIFEXITED(child_status)) {
-		/* Happy path (but also sad path sometimes). */
-		error = WEXITSTATUS(child_status);
-		pr_debug("Child terminated with error code %d.", error);
-		return error;
-	}
+		error = create_pipes(fork_fds);
+		if (error)
+			return error;
+
+		/* We need to fork because execvp() magics the thread away. */
+		child_pid = fork();
+		if (child_pid == 0) {
+			/* This code is run by the child. */
+			handle_child_thread(uri, is_ta, fork_fds);
+		}
+
+		/* This code is run by us. */
+		error = read_pipes(fork_fds);
+		if (error)
+			return error;
+
+		error = waitpid(child_pid, &child_status, 0);
+		do {
+			if (error == -1) {
+				error = errno;
+				pr_err("The rsync sub-process returned error %d (%s)",
+				    error, strerror(error));
+				if (child_status > 0)
+					break;
+				return error;
+			}
+		} while (0);
+
+		if (WIFEXITED(child_status)) {
+			/* Happy path (but also sad path sometimes). */
+			error = WEXITSTATUS(child_status);
+			pr_debug("Child terminated with error code %d.", error);
+			if (!error)
+				return 0;
+			if (retries == config_get_rsync_retry_count()) {
+				pr_info("Max RSYNC retries (%u) reached on '%s', won't retry again.",
+				    retries, uri_get_global(uri));
+				return error;
+			}
+			pr_info("Retrying RSYNC '%s' in %u seconds, %u attempts remaining.",
+			    uri_get_global(uri),
+			    config_get_rsync_retry_interval(),
+			    config_get_rsync_retry_count() - retries);
+			retries++;
+			sleep(config_get_rsync_retry_interval());
+			continue;
+		}
+		break;
+	} while (true);
 
 	if (WIFSIGNALED(child_status)) {
 		switch (WTERMSIG(child_status)) {
@@ -358,7 +404,8 @@ do_rsync(struct rpki_uri *uri, bool is_ta)
 			pr_err("Killed.");
 			break;
 		default:
-			pr_err("The RSYNC was terminated by a signal I don't have a handler for. Dunno; guess I'll just die.");
+			pr_err("The RSYNC was terminated by a signal [%d] I don't have a handler for. Dunno; guess I'll just die.",
+			    WTERMSIG(child_status));
 			break;
 		}
 		return -EINTR; /* Meh? */
@@ -382,21 +429,21 @@ download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
 	 * Note:
 	 * @requested_uri is the URI we were asked to RSYNC.
 	 * @rsync_uri is the URL we're actually going to RSYNC.
-	 * (They can differ, depending on config_get_sync_strategy().)
+	 * (They can differ, depending on config_get_rsync_strategy().)
 	 */
 	struct validation *state;
 	struct uri_list *visited_uris;
 	struct rpki_uri *rsync_uri;
 	int error;
 
-	if (config_get_sync_strategy() == SYNC_OFF)
+	if (!config_get_rsync_enabled())
 		return 0;
 
 	state = state_retrieve();
 	if (state == NULL)
 		return -EINVAL;
 
-	visited_uris = validation_visited_uris(state);
+	visited_uris = validation_rsync_visited_uris(state);
 
 	if (!force && is_already_downloaded(requested_uri, visited_uris)) {
 		pr_debug("No need to redownload '%s'.",
@@ -423,3 +470,24 @@ download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
 	uri_refput(rsync_uri);
 	return error;
 }
+
+void
+reset_downloaded(void)
+{
+	struct validation *state;
+	struct uri_list *list;
+	struct uri *uri;
+
+	state = state_retrieve();
+	if (state == NULL)
+		return;
+
+	list = validation_rsync_visited_uris(state);
+
+	while (!SLIST_EMPTY(list)) {
+		uri = SLIST_FIRST(list);
+		SLIST_REMOVE_HEAD(list, next);
+		uri_refput(uri->uri);
+		free(uri);
+	}
+}
