@@ -51,13 +51,15 @@ struct state {
 	 * during the current iteration.)
 	 */
 	struct db_table *base;
-	/** ROA changes to @base over time. */
+	/** DB changes to @base over time. */
 	struct deltas_db deltas;
 
+	/* Last valid SLURM applied to base */
+	struct db_slurm *slurm;
+
 	serial_t next_serial;
 	uint16_t v0_session_id;
 	uint16_t v1_session_id;
-	time_t last_modified_date;
 } state;
 
 /** Read/write lock, which protects @state and its inhabitants. */
@@ -95,20 +97,26 @@ vrps_init(void)
 	    ? (state.v0_session_id - 1)
 	    : (0xFFFFu);
 
+	state.slurm = NULL;
+
 	error = pthread_rwlock_init(&state_lock, NULL);
 	if (error) {
-		deltas_db_cleanup(&state.deltas, deltagroup_cleanup);
-		return pr_errno(error, "state pthread_rwlock_init() errored");
+		error = pr_errno(error, "state pthread_rwlock_init() errored");
+		goto release_deltas;
 	}
 
 	error = pthread_rwlock_init(&table_lock, NULL);
 	if (error) {
-		pthread_rwlock_destroy(&state_lock);
-		deltas_db_cleanup(&state.deltas, deltagroup_cleanup);
-		return pr_errno(error, "table pthread_rwlock_init() errored");
+		error = pr_errno(error, "table pthread_rwlock_init() errored");
+		goto release_state_lock;
 	}
 
 	return 0;
+release_state_lock:
+	pthread_rwlock_destroy(&state_lock);
+release_deltas:
+	deltas_db_cleanup(&state.deltas, deltagroup_cleanup);
+	return error;
 }
 
 void
@@ -116,6 +124,8 @@ vrps_destroy(void)
 {
 	if (state.base != NULL)
 		db_table_destroy(state.base);
+	if (state.slurm != NULL)
+		db_slurm_destroy(state.slurm);
 	deltas_db_cleanup(&state.deltas, deltagroup_cleanup);
 	/* Nothing to do with error codes from now on */
 	pthread_rwlock_destroy(&state_lock);
@@ -129,6 +139,13 @@ vrps_destroy(void)
 	rwlock_unlock(lock);						\
 	return error;
 
+#define RLOCK_HANDLER(lock, cb)						\
+	int error;							\
+	rwlock_read_lock(lock);						\
+	error = cb;							\
+	rwlock_unlock(lock);						\
+	return error;
+
 int
 handle_roa_v4(uint32_t as, struct ipv4_prefix const *prefix,
     uint8_t max_length, void *arg)
@@ -254,8 +271,8 @@ vrps_purge(struct deltas **deltas)
 	return resize_deltas_db(&state.deltas, group);
 }
 
-int
-vrps_update(bool *changed)
+static int
+__vrps_update(bool *changed)
 {
 	struct db_table *old_base;
 	struct db_table *new_base;
@@ -274,16 +291,11 @@ vrps_update(bool *changed)
 
 	rwlock_write_lock(&state_lock);
 
-	/*
-	 * TODO (next iteration) Remember the last valid SLURM
-	 *
-	 * Currently SLURM is ignored if it has errors, the error is logged and
-	 * the new_base isn't altered. Instead of this, the last valid SLURM
-	 * should be remembered, and will be applied when a new SLURM has
-	 * errors; a warning should be logged to indicate which version of the
-	 * SLURM is being applied.
-	 */
-	slurm_apply(&new_base);
+	error = slurm_apply(&new_base, &state.slurm);
+	if (error) {
+		rwlock_unlock(&state_lock);
+		goto revert_base;
+	}
 
 	if (state.base != NULL) {
 		error = compute_deltas(state.base, new_base, &deltas);
@@ -325,7 +337,8 @@ vrps_update(bool *changed)
 		if (db_table_roa_count(new_base) +
 		    db_table_router_key_count(new_base) == 0) {
 			rwlock_unlock(&state_lock);
-			return 0;
+			error = 0; /* OK (said explicitly) */
+			goto revert_base;
 		}
 		error = create_empty_delta(&deltas);
 		if (error) {
@@ -357,6 +370,60 @@ revert_base:
 	return error;
 }
 
+int
+vrps_update(bool *changed)
+{
+	time_t start, finish;
+	long int exec_time;
+	serial_t serial;
+	int error;
+
+	/*
+	 * This wrapper is mainly for log informational data, so if there's no
+	 * need don't do unnecessary calls
+	 */
+	if (!log_info_enabled())
+		return __vrps_update(changed);
+
+	pr_info("Starting validation.");
+	serial = START_SERIAL;
+	if (config_get_mode() == SERVER) {
+		error = get_last_serial_number(&serial);
+		if (!error)
+			pr_info("- Current serial number is %u.", serial);
+	}
+
+	time(&start);
+	error = __vrps_update(changed);
+	time(&finish);
+	exec_time = finish - start;
+
+	pr_info("Validation finished:");
+	rwlock_read_lock(&state_lock);
+	do {
+		if (state.base == NULL) {
+			rwlock_unlock(&state_lock);
+			pr_info("- Valid Prefixes: 0");
+			pr_info("- Valid Router Keys: 0");
+			if (config_get_mode() == SERVER)
+				pr_info("- No serial number.");
+			break;
+		}
+
+		pr_info("- Valid Prefixes: %u", db_table_roa_count(state.base));
+		pr_info("- Valid Router Keys: %u",
+		    db_table_router_key_count(state.base));
+		if (config_get_mode() == SERVER) {
+			pr_info("- %s serial number is %u.",
+			    serial == state.next_serial - 1 ? "Current" : "New",
+			    state.next_serial - 1);
+		}
+		rwlock_unlock(&state_lock);
+	} while(0);
+	pr_info("- Real execution time: %ld secs.", exec_time);
+
+	return error;
+}
 /**
  * Please keep in mind that there is at least one errcode-aware caller. The most
  * important ones are
