@@ -86,10 +86,12 @@ create_server_socket(int *result)
 	struct addrinfo *addrs;
 	struct addrinfo *addr;
 	unsigned long port;
+	int reuse;
 	int fd; /* "file descriptor" */
 	int error;
 
 	*result = 0; /* Shuts up gcc */
+	reuse = 1;
 
 	error = init_addrinfo(&addrs);
 	if (error)
@@ -107,6 +109,20 @@ create_server_socket(int *result)
 			continue;
 		}
 
+		/* enable SO_REUSEADDR */
+		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &reuse,
+		    sizeof(int)) < 0) {
+			pr_errno(errno, "setsockopt(SO_REUSEADDR) failed");
+			continue;
+		}
+
+		/* enable SO_REUSEPORT */
+		if (setsockopt(fd, SOL_SOCKET, SO_REUSEPORT, &reuse,
+		    sizeof(int)) < 0) {
+			pr_errno(errno, "setsockopt(SO_REUSEPORT) failed");
+			continue;
+		}
+
 		if (bind(fd, addr->ai_addr, addr->ai_addrlen) < 0) {
 			pr_errno(errno, "bind() failed");
 			continue;
@@ -218,6 +234,14 @@ end_client(int fd)
 		print_close_failure(errno, fd);
 }
 
+static void
+print_client_addr(struct sockaddr_storage *addr, char const *action, int fd)
+{
+	char buffer[INET6_ADDRSTRLEN];
+	pr_info("Client %s [ID %d]: %s", action, fd,
+	    sockaddr2str(addr, buffer));
+}
+
 /*
  * The client socket threads' entry routine.
  * @arg must be released.
@@ -250,6 +274,7 @@ client_thread_cb(void *arg)
 			break;
 	}
 
+	print_client_addr(&param.addr, "closed", param.fd);
 	end_client(param.fd);
 	clients_forget(param.fd);
 
@@ -288,11 +313,7 @@ handle_client_connections(int server_fd)
 			return -EINVAL;
 		}
 
-		if (log_debug_enabled()) {
-			char buffer[INET6_ADDRSTRLEN];
-			pr_debug("Client accepted: %s",
-			    sockaddr2str(&client_addr, buffer));
-		}
+		print_client_addr(&client_addr, "accepted", client_fd);
 
 		/*
 		 * Note: My gut says that errors from now on (even the unknown
@@ -331,9 +352,10 @@ handle_client_connections(int server_fd)
  * Receive @arg to be called as a clients_foreach_cb
  */
 static int
-kill_client(struct client const *client, void *arg)
+kill_client(struct client *client, void *arg)
 {
 	end_client(client->fd);
+	print_client_addr(&(client->addr), "terminated", client->fd);
 	/* Don't call clients_forget to avoid deadlock! */
 	return 0;
 }
@@ -376,8 +398,6 @@ rtr_listen(void)
 		if (error)
 			pr_err("Error %d while trying to update the ROA database.",
 			    error);
-		else
-			pr_info("Validation finished.");
 		goto revert_clients_db; /* Error 0 it's ok */
 	}
 
